//--------------------------------------------------------------
// Fichero:  angvect.js
// Objetivo: Ángulo entre vectores
// Fecha:    M.14.8.2018
// Autor:    Pedro Reina <pedro@pedroreina.net>
// Licencia: CC0 1.0 Universal
//           http://creativecommons.org/publicdomain/zero/1.0/
//--------------------------------------------------------------

//----------------------------
// Variables globales
//----------------------------

// Datos que proporciona el usuario
NombreDato = Array ("X1", "X2", "Y1", "Y2");
var X1 = null;
var Y1 = null;
var X2 = null;
var Y2 = null;

// Respuestas que calcula el programa
var NombreRespuesta = Array ("Modulo1", "Modulo2", "Producto", "Coseno",
                              "AnguloRad", "AnguloGms");
var Modulo1 = null;
var Modulo2 = null;
var Producto = null;
var Coseno = null;
var AnguloRad = null;
var AnguloGms = null;

// Los controles HTML
var InputX1 = null;
var InputY1 = null;
var InputX2 = null;
var InputY2 = null;
var InputModulo1 = null;
var InputModulo2 = null;
var InputProducto = null;
var InputCoseno = null;
var InputAnguloRad = null;
var InputAnguloGms = null;

//----------------------------
// Funciones que debe implementar el programa
//----------------------------

//--------------------------------------------------------------
// Obtener los identificadores adicionales de los controles HTML
function ObtenIdentificadoresProblema()
  {  }

//--------------------------------------------------------------
// Colocar el foco en el primer elemento
function FocoAlPrimero()
  { InputX1.focus(); }

//--------------------------------------------------------------
// Obtener los datos que ha escrito el usuario
function ObtenDatos()
  {
  X1 = parseFloat(InputX1.value);
  Y1 = parseFloat(InputY1.value);
  X2 = parseFloat(InputX2.value);
  Y2 = parseFloat(InputY2.value);
  }

//--------------------------------------------------------------
// Comprobación de los datos
function CompruebaDatos()
  {
  Error = false;
  Mensaje = "Los datos no son válidos:\n"

  if ( NoNumero (NombreDato) )
    {
    Error = true;
    Mensaje += "\nHay datos no definidos";
    }

  if ( (X1 == 0 && Y1 == 0) || (X2 == 0 && Y2 == 0) )
    {
    Error = true;
    Mensaje += "\nNinguno de los vectores puede ser 0";
    }

  if ( Error )
    {
    alert (Mensaje);
    return false;
    }
  else
    { return true; }
  }

//--------------------------------------------------------------
// Calcular las respuestas
function CalculaRespuestas()
  {
  Modulo1 = Math.sqrt(X1*X1 + Y1*Y1);
  Modulo2 = Math.sqrt(X2*X2 + Y2*Y2);
  Producto = X1*X2 + Y1*Y2;
  Coseno = Producto / (Modulo1 * Modulo2);
  AnguloRad = Math.acos(Coseno);
  AnguloGms = gms(AnguloRad);
  }

//--------------------------------------------------------------
// La realización del dibujo
function RealizaDibujo()
  {
  // Dibujamos los ejes de coordenadas
  Contexto.fillRect (0.05*AnchuraDibujo, AlturaDibujo/2, 0.9*AnchuraDibujo, 2);
  Contexto.fillRect (AnchuraDibujo/2, 0.05*AlturaDibujo, 2, 0.9*AlturaDibujo);

  // Calculamos la escala
  var EscX = 0.45 * AnchuraDibujo / Math.max (Math.abs(X1), Math.abs(X2));
  var EscY = 0.45 * AlturaDibujo / Math.max (Math.abs(Y1), Math.abs(Y2));
  Escala = Math.min(EscX, EscY);

  // Ponemos el punto (0,0) en el centro de la zona de dibujo
  Contexto.translate(AnchuraDibujo/2, AlturaDibujo/2);

  // En canvas las ordenadas son positivas hacia abajo
  Y1 = -Y1; Y2 = -Y2;

  // Trazo medio
  Contexto.lineWidth = 3;

  // Color azul
  Contexto.strokeStyle = "#0000CC";

  // Los vectores
  Flecha(Contexto, esc(X1), esc(Y1));
  Flecha(Contexto, esc(X2), esc(Y2));

  // Color rojo
  Contexto.strokeStyle = "#CC0000";

  // El arco del ángulo
  var Radio = Math.min (Modulo1, Modulo2) / 6;
  var Angulo1 = Math.atan2 (Y1, X1);
  var Angulo2 = Math.atan2 (Y2, X2);
  var ComienzoArco = Math.min (Angulo1, Angulo2);
  var FinalArco = Math.max (Angulo1, Angulo2);
  var Sentido = (FinalArco-ComienzoArco>Math.PI) ? true : false;
  Contexto.beginPath();
  Contexto.arc (0, 0, esc(Radio), ComienzoArco, FinalArco, Sentido);
  Contexto.stroke();
  }

//----------------------------
// Funciones auxiliares
//----------------------------

//--------------------------------------------------------------
// Pasar un ángulo en radianes a grados, minutos y segundos
function gms(angulo)
  {
  angulo = angulo / Math.PI * 180;
  var grado = Math.floor (angulo);
  var minuto = 60 * (angulo-grado);
  var segundo = 60 * (minuto-Math.floor(minuto));
  minuto = Math.floor (minuto);
  segundo = Math.floor (10000 * segundo + 0.5) / 10000;
  return ( grado + "\u00B0 " + minuto + "\' " + segundo + "\"");
  }

//--------------------------------------------------------------
// Dibujo de flechas. Autor: Titus Cieslewski; adaptación de Pedro Reina
// http://stackoverflow.com/questions/808826/draw-arrow-on-canvas-tag
function Flecha(c, HastaX, HastaY)
  {
  var Longitud = 10;   // Longitud de la flecha
  var Angulo = Math.atan2 (HastaY, HastaX);

  c.beginPath();

  c.moveTo (0, 0);
  c.lineTo (HastaX, HastaY);
  c.lineTo (HastaX - Longitud*Math.cos(Angulo-Math.PI/6),
            HastaY - Longitud*Math.sin(Angulo-Math.PI/6));
  c.moveTo (HastaX, HastaY);
  c.lineTo (HastaX - Longitud*Math.cos(Angulo+Math.PI/6),
            HastaY - Longitud*Math.sin(Angulo+Math.PI/6));

  c.stroke();
  }
