//--------------------------------------------------------------
// Fichero:  binomial.js
// Objetivo: Cálculos con la distribución binomial
// Fecha:    M.14.8.2018
// Autor:    Pedro Reina <pedro@pedroreina.net>
// Licencia: CC0 1.0 Universal
//           http://creativecommons.org/publicdomain/zero/1.0/
//--------------------------------------------------------------

//----------------------------
// Variables globales
//----------------------------

// Datos que proporciona el usuario
var NombreDato = Array ("NumRep", "ProbExito", "Menor", "Mayor");
var NumRep = 0;
var ProbExito = 0;
var Menor = 0;
var Mayor = 0;

// Valores que calcula el programa
var NombreRespuesta = Array ("Probabilidad", "Media", "Desviacion");
var Probabilidad = 0;
var Media = 0;
var Desviacion = 0;

// El array con la probabilidad de cada número de éxitos
var ConjuntoProb = null;

// Los controles HTML
var InputNumRep = null;
var InputProbExito = null;
var InputMenor = null;
var InputMayor = null;
var InputProbabilidad = null;
var InputMedia = null;
var InputDesviacion = null;

//----------------------------
// Funciones que debe implementar el programa
//----------------------------

//--------------------------------------------------------------
// Obtener los identificadores adicionales de los controles HTML
function ObtenIdentificadoresProblema()
  {  }

//--------------------------------------------------------------
// Colocar el foco en el primer elemento
function FocoAlPrimero()
  { InputNumRep.focus(); }

//--------------------------------------------------------------
// Obtener los datos que ha escrito el usuario
function ObtenDatos()
  {
  NumRep = parseInt(InputNumRep.value);
  ProbExito = parseFloat(InputProbExito.value);
  Menor = parseInt(InputMenor.value);
  Mayor = parseInt(InputMayor.value);
  }

//--------------------------------------------------------------
// Comprobación de los datos
function CompruebaDatos()
  {
  Error = false;
  Mensaje = "Los datos no son válidos:\n"

  if ( NoNumero (NombreDato) )
    {
    Error = true;
    Mensaje += "\nHay datos no definidos";
    }

  if ( NumRep > 269 )
    {
    Error = true;
    Mensaje += "\nEl número de repeticiones no puede ser mayor que 269";
    }

  if ( NumRep <= 0 )
    {
    Error = true;
    Mensaje += "\nEl número de repeticiones debe ser positivo";
    }

  if ( ProbExito < 0 || ProbExito > 1 )
    {
    Error = true;
    Mensaje += "\nLa probabilidad de éxito debe estar entre 0 y 1";
    }

  if ( Menor < 0 )
    {
    Error = true;
    Mensaje += "\nEl extremo menor debe ser mayor o igual a 0";
    }

  if ( Mayor > NumRep )
    {
    Error = true;
    Mensaje += "\nEl extremo mayor debe ser menor o igual al número de rep.";
    }

  if ( Menor > Mayor )
    {
    Error = true;
    Mensaje += "\nEl extremo menor debe ser menor o igual que el mayor";
    }

  if ( Error )
    {
    alert (Mensaje);
    return false;
    }
  else
    { return true; }
  }

//--------------------------------------------------------------
// Calcular las respuestas
function CalculaRespuestas()
  {
  // Probabilidad de fracaso en una repetición
  var ProbFracaso = 1-ProbExito;

  // Probabilidades de cada posible número de éxitos
  ConjuntoProb = Array (NumRep+1);
  for ( var i=0 ; i<= NumRep ; i++ )
    {
    ConjuntoProb[i] = Comb (NumRep, i) *
      Math.pow (ProbExito, i) * Math.pow (ProbFracaso, NumRep-i);
    }

  // Probabilidad pedida
  Probabilidad = 0;
  for ( var i=Menor ; i<=Mayor ; i++ )
    { Probabilidad += ConjuntoProb[i]; }

  Media = NumRep * ProbExito;
  Desviacion = Math.sqrt (Media*ProbFracaso);
  }

//--------------------------------------------------------------
// La realización del dibujo
function RealizaDibujo()
  {
  // Calculamos la escala necesaria para mostrar en el dibujo
  // las probabilidades ocupando el espacio disponible
  Escala = 0.9 * AlturaDibujo / Math.max.apply (Math, ConjuntoProb);

  // El punto más bajo a la izquierda en que empezamos a dibujar
  var OrigenX = 0.05*AnchuraDibujo;
  var OrigenY = 0.95*AlturaDibujo;

  // La anchura de cada Barra
  var AnchuraBarra = 0.9 * AnchuraDibujo / (2 * NumRep + 1);

  // Para cada valor
  for ( var i=0 ; i<=NumRep ; i++ )
    {
    // Elegimos el color de la barra
    if ( i<Menor || i>Mayor )
      { Contexto.fillStyle = '#BBB'}
    else
      { Contexto.fillStyle = '#700'}

    // Dibujamos una barra
    var AlturaBarra = Math.max (1, esc(ConjuntoProb[i]));
    Contexto.fillRect (OrigenX + 2*AnchuraBarra*i, OrigenY - AlturaBarra,
                       AnchuraBarra, AlturaBarra);
    }
  }

//----------------------------
// Funciones auxiliares
//----------------------------

//--------------------------------------------------------------
// Combinaciones sin repetición de m elementos tomados n a n
function Comb(m, n)
  {
  var Respuesta = 1;

  if ( n > m/2 )
    { n = m - n; }

  var Factor = m;
  for ( var i=0 ; i<n ; i++ )
    { Respuesta *= Factor--; }

  for ( var i=n ; i>1 ; i-- )
    { Respuesta /= i; }

  return Respuesta;
  }
