//--------------------------------------------------------------
// Fichero:  bolzano.js
// Objetivo: Aplicar el teorema de Bolzano
// Fecha:    M.14.8.2018
// Autor:    Pedro Reina <pedro@pedroreina.net>
// Licencia: CC0 1.0 Universal
//           http://creativecommons.org/publicdomain/zero/1.0/
//--------------------------------------------------------------

//----------------------------
// Variables globales
//----------------------------

// Datos que proporciona el usuario
var NombreDato = Array ("Expresion", "Izquierdo", "Derecho", "Precision");
var NombreDatoNumero = Array ("Izquierdo", "Derecho", "Precision");
var Expresion = null;
var Izquierdo = null;
var Derecho = null;
var Precision = null;

// Valores que calcula el programa
var NombreRespuesta = Array ("Raiz", "Iteraciones");
var Raiz = null;
var Iteraciones = null;

// Los controles HTML
var InputExpresion = null;
var InputIzquierdo = null;
var InputDerecho = null;
var InputPrecision = null;
var InputRaiz = null;
var InputIteraciones = null;

//----------------------------
// Funciones que debe implementar el programa
//----------------------------

//--------------------------------------------------------------
// Obtener los identificadores adicionales de los controles HTML
function ObtenIdentificadoresProblema()
  {  }

//--------------------------------------------------------------
// Colocar el foco en el primer elemento
function FocoAlPrimero()
  { InputExpresion.focus(); }

//--------------------------------------------------------------
// Obtener los datos que ha escrito el usuario
function ObtenDatos()
  {
  Expresion = InputExpresion.value;
  Izquierdo = parseFloat(InputIzquierdo.value);
  Derecho = parseFloat(InputDerecho.value);
  Precision = parseFloat(InputPrecision.value);
  }

//--------------------------------------------------------------
// Comprobación de los datos
function CompruebaDatos()
  {
  Error = false;
  Mensaje = "Los datos no son válidos:\n"

  if ( NoNumero (NombreDatoNumero) )
    {
    Error = true;
    Mensaje += "\nHay datos no definidos";
    }

  if ( Izquierdo > Derecho )
    {
    Error = true;
    Mensaje += "\nEl extremo izquierdo debe ser menor que el derecho";
    }

  if ( Error )
    {
    alert (Mensaje);
    return false;
    }
  else
    { return true; }
  }

//--------------------------------------------------------------
// Calcular las respuestas
function CalculaRespuestas()
  {
  // Los puntos en los que la función es positiva y negativa
  var Positiva = null; var Negativa = null;

  // El punto medio del segmento
  var Medio = null;

  // Modificamos la expresión de la función
  ModificaExpresion();

  // La función debe tener distinto signo no nulo en los extremos
  if ( (f(Izquierdo) * f(Derecho)) >= 0 )
    {
    Raiz = "Teorema no aplicable";
    Iteraciones = "";
    }

  else
    {
    // Preparamos los datos iniciales antes de iterar
    if ( f(Izquierdo) > 0 )
      {
      Positiva = Izquierdo;
      Negativa = Derecho;
      }
    else
      {
      Positiva = Derecho;
      Negativa = Izquierdo;
      }

    // Aplicamos reiteradamente el teorema
    Iteraciones = 0;
    var Sigue = true;
    while ( Math.abs (Positiva-Negativa) > Precision && Sigue )
      {
      Iteraciones++;
      Medio = (Positiva+Negativa) / 2;
      if ( f(Medio) == 0 )
        {
        Raiz = Medio;
        Sigue = false;
        }
      else
        {
        if ( f(Medio) > 0 )  { Positiva = Medio; }
        else                 { Negativa = Medio; }
        }
      }
    if ( Sigue )  { Raiz = (Positiva+Negativa) / 2; }
    }
  }

//--------------------------------------------------------------
// La realización del dibujo
function RealizaDibujo()
  {
  // La abscisa y la ordenada en coordenadas del canvas
  var Abscisa = null; var Ordenada = null;

  // La abscisa y la ordenada en la función
  var x = null; var y = null;

  // Calculamos la escala para que se vean bien los extremos
  Escala = 0.45 * AlturaDibujo /
           Math.max (Math.abs(f(Izquierdo)), Math.abs(f(Derecho)));

  // El punto más a la izquierda en que empezamos a dibujar
  var OrigenX = 0.05*AnchuraDibujo;
  var OrigenY = 0.5*AlturaDibujo;

  // Dibujamos el eje de abscisas
  Contexto.fillRect (0.05*AnchuraDibujo, AlturaDibujo/2, 0.9*AnchuraDibujo, 2);

  // Trazo medio y color azul para la función
  Contexto.lineWidth = 3;
  Contexto.strokeStyle = "#0000CC";

  // Comienza el dibujo de la función
  Contexto.beginPath();
  var Primero = true;

  // Para cada píxel horizontal
  for ( Abscisa = OrigenX ; Abscisa <= OrigenX + 0.9*AnchuraDibujo ; Abscisa++ )
    {
    // Calculamos la altura que hay que dibujar
    x = (Abscisa-OrigenX) / (0.9*AnchuraDibujo/(Derecho-Izquierdo)) + Izquierdo;
    y = f(x);
    Ordenada = esc(y);

    if ( Primero )
      {
      Contexto.moveTo (Abscisa, OrigenY-Ordenada);
      Primero = false;
      }
    else
      { Contexto.lineTo (Abscisa, OrigenY-Ordenada); }
    }

  // Termina el dibujo
  Contexto.stroke();

  // Marcamos los extremos
  Contexto.fillStyle="#070";
  Contexto.beginPath();
  Contexto.arc (OrigenX, OrigenY-esc(f(Izquierdo)), 4, 0, 2*Math.PI, true);
  Contexto.closePath();
  Contexto.fill();
  Contexto.beginPath();
  Contexto.arc (OrigenX+0.9*AnchuraDibujo, OrigenY-esc(f(Derecho)),
                4, 0, 2*Math.PI, true);
  Contexto.closePath();
  Contexto.fill();

  // Si hemos encontrado una raíz, la marcamos con un punto
  if ( typeof (Raiz) == "number" )
    {
    Abscisa = OrigenX+((Raiz-Izquierdo)/(Derecho-Izquierdo))*0.9*AnchuraDibujo;
    Contexto.beginPath();
    Contexto.fillStyle="#700";
    Contexto.arc (Abscisa, OrigenY, 4, 0, 2*Math.PI, true);
    Contexto.closePath();
    Contexto.fill();
    }
  }
