//--------------------------------------------------------------
// Fichero:  intdef.js
// Objetivo: Calcular y visualizar una integral definida
// Fecha:    D.16.7.2017
// Autor:    Pedro Reina
// Licencia: CC0 1.0 Universal
//           http://creativecommons.org/publicdomain/zero/1.0/
//--------------------------------------------------------------

//------------------------------------------
// Variables globales
//------------------------------------------

// Datos que proporciona el usuario
var NombreDato = Array ("Expresion", "Izquierdo", "Derecho");
var NombreDatoNumero = Array ("Izquierdo", "Derecho");
var Expresion = null;
var Izquierdo = null;
var Derecho = null;
var PuntoIntervalo = Array ("minimo", "maximo", "izquierdo", "derecho",
                            "aleatorio");
var TipoPunto = null;
var NumeroIntervalos = null;

// Valores que calcula el programa
var NombreRespuesta = Array ("Suma");
var Suma = null;

// Los controles HTML de la integral
var InputExpresion = null;
var InputIzquierdo = null;
var InputDerecho = null;
var InputSuma = null;

// Los controles HTML del interfaz de usuario
var Contexto = null;
var BotonPuntoIntervalo = null;
var Deslizador = null;

// Los valores de las abscisas y las ordenadas
var ValorAbscisa = Array();
var ValorOrdenada = Array();
var ValorElegido = Array();

// La anchura de cada intervalo
var Paso = null;

//----------------------------
// Funciones que debe implementar el programa
//----------------------------

//--------------------------------------------------------------
// Obtener los identificadores adicionales de los controles HTML
function ObtenIdentificadoresProblema()
  {
  // Averiguamos el nombre del grupo de botones para elegir el punto
  BotonPuntoIntervalo = document.getElementsByName('tipo');

  // Averiguamos el id del deslizador
  Deslizador = document.getElementById("deslizador");
  }

//--------------------------------------------------------------
// Colocar el foco en el primer elemento
function FocoAlPrimero()
  { InputExpresion.focus(); }

//--------------------------------------------------------------
// Obtener los datos que ha escrito el usuario
function ObtenDatos()
  {
  Expresion = InputExpresion.value;
  Izquierdo = parseFloat(InputIzquierdo.value);
  Derecho = parseFloat(InputDerecho.value);
  }

//--------------------------------------------------------------
// Comprobación de los datos
function CompruebaDatos()
  {
  Error = false;
  Mensaje = "Los datos no son válidos:\n"

  if ( NoNumero (NombreDatoNumero) )
    {
    Error = true;
    Mensaje += "\nHay datos no definidos";
    }

  if ( Izquierdo > Derecho )
    {
    Error = true;
    Mensaje += "\nEl extremo izquierdo debe ser menor que el derecho";
    }

  if ( Error )
    {
    alert (Mensaje);
    return false;
    }
  else
    { return true; }
  }

//--------------------------------------------------------------
// Calcular las respuestas
function CalculaRespuestas()
  {
  // Modificamos la expresión de la función
  ModificaExpresion();

  // Obtenemos el punto de cada intervalo que hay que usar
  for ( var i=0 ; i<BotonPuntoIntervalo.length ; i++ )
    {
    if ( BotonPuntoIntervalo[i].checked )
      { TipoPunto = PuntoIntervalo[i]; }
    }

  // Obtenemos cuántos intervalos hay que utilizar
  NumeroIntervalos = Deslizador.value;

  // Calculamos la anchura de los intervalos
  // (Usamos la definición de Cauchy, no la de Riemman)
  Paso = (Derecho-Izquierdo) / NumeroIntervalos;

  // Calculamos todos los valores de las abscisas necesarios
  // Si hay n intervalos, habrá n+1 valores
  ValorAbscisa = Array();
  for ( var i=0 ; i<=NumeroIntervalos ; i++ )
    { ValorAbscisa.push(Izquierdo+i*Paso); }

  // Calculamos todos los valores de las ordenadas correspondientes
  ValorOrdenada = Array();
  for ( var i=0 ; i<=NumeroIntervalos ; i++ )
    { ValorOrdenada.push(f(ValorAbscisa[i])); }

  // Ponemos a cero el contador de la suma
  Suma = 0;

  // Variables auxiliares para este bloque de código
  var PuntoIzquierdo = null;
  var ValorEnIzquierdo = null;
  var PuntoDerecho = null;
  var ValorEnDerecho = null;
  var Abscisa = null;
  var Ordenada = null;

  // Vamos sumando lo que da cada intervalo
  // y nos quedamos con el valor elegido para la ordenada
  ValorElegido = Array();
  for ( var i=0 ; i<NumeroIntervalos ; i++ )
    {
    // Elegimos el punto que vamos a usar en este intervalo
    PuntoIzquierdo = ValorAbscisa[i];
    PuntoDerecho = ValorAbscisa[i+1];
    if ( TipoPunto == "izquierdo" )
      { Abscisa = PuntoIzquierdo; }
    if ( TipoPunto == "derecho" )
      { Abscisa = PuntoDerecho; }
    if ( TipoPunto == "aleatorio" )
      { Abscisa = PuntoIzquierdo + Math.random()*Paso; }
    // Atención: chapuza inmensa, solo uso dos valores para hallar el mínimo
    if ( TipoPunto == "minimo" )
      {
      ValorEnIzquierdo = ValorOrdenada[i];
      ValorEnDerecho = ValorOrdenada[i+1];
      if ( ValorEnIzquierdo < ValorEnDerecho )
        { Abscisa = PuntoIzquierdo; }
      else
        { Abscisa = PuntoDerecho; }
      }
    // Atención: chapuza inmensa, solo uso dos valores para hallar el máximo
    if ( TipoPunto == "maximo" )
      {
      ValorEnIzquierdo = ValorOrdenada[i];
      ValorEnDerecho = ValorOrdenada[i+1];
      if ( ValorEnIzquierdo > ValorEnDerecho )
        { Abscisa = PuntoIzquierdo; }
      else
        { Abscisa = PuntoDerecho; }
      }

    // Averiguamos el valor de la función en ese punto
    Ordenada = f(Abscisa);

    // Y nos lo quedamos
    ValorElegido.push(Ordenada);

    // Sumamos este rectángulo (con signo)
    Suma += Paso * Ordenada;
    }
  }

//--------------------------------------------------------------
// La realización del dibujo
function RealizaDibujo()
  {
  // La abscisa y la ordenada en coordenadas del canvas
  var Abscisa = null; var Ordenada = null;

  // La abscisa y la ordenada en la función
  var x = null; var y = null;

  // Calculamos la escala para que se vean bien todos los valores
  var ValorMaximo = Math.max.apply(null, ValorOrdenada);
  var ValorMinimo = Math.min.apply(null, ValorOrdenada);
  if ( ValorMaximo>0 && ValorMinimo>0 )
    { Escala = 0.9 * AlturaDibujo / ValorMaximo; }
  else if ( ValorMaximo<0 && ValorMinimo<0 )
    { Escala = 0.9 * AlturaDibujo / (-ValorMinimo); }
  else
    { Escala = 0.9 * AlturaDibujo / Math.abs(ValorMaximo-ValorMinimo); }

  // El punto más a la izquierda en que empezamos a dibujar
  var OrigenX = 0.05*AnchuraDibujo;
  var OrigenY = null;
  if ( ValorMaximo>0 && ValorMinimo>0 )
    { OrigenY = 0.95*AlturaDibujo; }
  else if ( ValorMaximo<0 && ValorMinimo<0 )
    { OrigenY = 0.05*AlturaDibujo; }
  else
    { OrigenY = 0.05*AlturaDibujo+esc(ValorMaximo); }

  // Dibujamos los rectángulos
  var Altura;
  var Anchura = 0.9*AnchuraDibujo / NumeroIntervalos;
  for ( var i=0 ; i<NumeroIntervalos ; i++ )
    {
    if ( i%2 )
      { Contexto.fillStyle="#ffb380"; }
    else
      { Contexto.fillStyle="#ff944d"; }
    Altura = -esc(ValorElegido[i]);
    Contexto.fillRect (OrigenX + Anchura*i, OrigenY, Anchura, Altura);
    }

  // Dibujamos el eje de abscisas
  Contexto.fillStyle="#000";
  Contexto.fillRect (OrigenX, OrigenY, 0.9*AnchuraDibujo, 2);

  // Trazo medio y color azul para la función
  Contexto.lineWidth = 3;
  Contexto.strokeStyle = "#0000CC";

  // Comienza el dibujo de la función
  Contexto.beginPath();
  var Primero = true;

  // Para cada píxel horizontal
  for ( Abscisa = OrigenX ; Abscisa <= OrigenX + 0.9*AnchuraDibujo ; Abscisa++ )
    {
    // Calculamos la altura que hay que dibujar
    x = (Abscisa-OrigenX) / (0.9*AnchuraDibujo/(Derecho-Izquierdo)) + Izquierdo;
    y = f(x);
    Ordenada = esc(y);

    if ( Primero )
      {
      Contexto.moveTo (Abscisa, OrigenY-Ordenada);
      Primero = false;
      }
    else
      { Contexto.lineTo (Abscisa, OrigenY-Ordenada); }
    }

  // Termina el dibujo
  Contexto.stroke();

  // Marcamos los extremos
  Contexto.fillStyle="#070";
  Contexto.beginPath();
  Contexto.arc (OrigenX, OrigenY-esc(f(Izquierdo)), 4, 0, 2*Math.PI, true);
  Contexto.closePath();
  Contexto.fill();
  Contexto.beginPath();
  Contexto.arc (OrigenX+0.9*AnchuraDibujo, OrigenY-esc(f(Derecho)),
                4, 0, 2*Math.PI, true);
  Contexto.closePath();
  Contexto.fill();
  }
