//--------------------------------------------------------------
// Fichero:  normal.js
// Objetivo: Cálculos con la distribución normal
// Fecha:    M.14.8.2018
// Autor:    Pedro Reina <pedro@pedroreina.net>
// Licencia: CC0 1.0 Universal
//           http://creativecommons.org/publicdomain/zero/1.0/
//--------------------------------------------------------------

//----------------------------
// Variables globales
//----------------------------

// Datos que proporciona el usuario
var NombreDato = Array ("Media", "Desviacion", "Menor", "Mayor");
var Media = 0;
var Desviacion = 0;
var Menor = 0;
var Mayor = 0;

// Valores que calcula el programa
var NombreRespuesta = Array ("Probabilidad");
var Probabilidad = 0;

// Los controles HTML
var InputMedia = null;
var InputDesviacion = null;
var InputMenor = null;
var InputMayor = null;
var InputProbabilidad = null;

//----------------------------
// Funciones que debe implementar el programa
//----------------------------

//--------------------------------------------------------------
// Obtener los identificadores adicionales de los controles HTML
function ObtenIdentificadoresProblema()
  {  }

//--------------------------------------------------------------
// Colocar el foco en el primer elemento
function FocoAlPrimero()
  { InputMedia.focus(); }

//--------------------------------------------------------------
// Obtener los datos que ha escrito el usuario
function ObtenDatos()
  {
  Media = parseFloat(InputMedia.value);
  Desviacion = parseFloat(InputDesviacion.value);
  Menor = parseFloat(InputMenor.value);
  Mayor = parseFloat(InputMayor.value);
  }

//--------------------------------------------------------------
// Comprobación de los datos
function CompruebaDatos()
  {
  Error = false;
  Mensaje = "Los datos no son válidos:\n"

  if ( NoNumero (NombreDato) )
    {
    Error = true;
    Mensaje += "\nHay datos no definidos";
    }

  if ( Desviacion <= 0 )
    {
    Error = true;
    Mensaje += "\nLa desviación debe ser positiva";
    }

  if ( Menor > Mayor )
    {
    Error = true;
    Mensaje += "\nEl extremo menor debe ser menor o igual que el mayor";
    }

  if ( Error )
    {
    alert (Mensaje);
    return false;
    }
  else
    { return true; }
  }

//--------------------------------------------------------------
// Calcular las respuestas
function CalculaRespuestas()
  {
  // Probabilidad pedida
  Probabilidad = n01 (tip(Mayor)) - n01 (tip(Menor));

  // Solo damos seis decimales, ya que el método no es más preciso
  Probabilidad = Math.round (1000000*Probabilidad) / 1000000;
  }

//--------------------------------------------------------------
// La realización del dibujo
function RealizaDibujo()
  {
  // La abscisa y la ordenada en coordenadas del canvas
  var Abscisa = null; var Ordenada = null;

  // La abscisa y la ordenada en N(0,1)
  var x = null; var y = null;

  // Tipificación de los valores Menor y Mayor
  var TipMenor = tip (Menor);
  var TipMayor = tip (Mayor);

  // Calculamos la escala
  Escala = 0.9 * AlturaDibujo / fd(0);

  // El punto más bajo a la izquierda en que empezamos a dibujar
  var OrigenX = 0.05*AnchuraDibujo;
  var OrigenY = 0.95*AlturaDibujo;

  // Dibujamos el eje de abscisas
  Contexto.fillRect (OrigenX, OrigenY, 0.9*AnchuraDibujo, 2);

  // Trazo medio y color azul para la función de densidad
  Contexto.lineWidth = 3;
  Contexto.strokeStyle = "#0000CC";

  // Comienza el dibujo de la función de densidad
  Contexto.beginPath();
  var Primero = true;

  // Para cada píxel horizontal
  for ( Abscisa = OrigenX ; Abscisa <= OrigenX + 0.9*AnchuraDibujo ; Abscisa++ )
    {
    // Calculamos la altura que hay que dibujar
    x = (Abscisa-OrigenX) / (0.9*AnchuraDibujo/6) - 3;
    y = fd (x);
    Ordenada = esc(y);

    // Coloreado de las zonas pedida y no pedidas
    // Elegimos el color de la barra
    if ( x<TipMenor || x>TipMayor )
      { Contexto.fillStyle = '#BBB'}
    else
      { Contexto.fillStyle = '#700'}
    // Dibujamos un barra muy fina
    Contexto.fillRect (Abscisa, OrigenY - Ordenada, 1, Ordenada);

    // La función de densidad
    if ( Primero )
      {
      Contexto.moveTo (Abscisa, OrigenY-Ordenada);
      Primero = false;
      }
    else
      { Contexto.lineTo (Abscisa, OrigenY-Ordenada); }
    }

  // Termina el dibujo
  Contexto.stroke();
  }

//----------------------------
// Funciones auxiliares
//----------------------------

//--------------------------------------------------------------
// Tipificación de una variable
function tip(x)
  { return (x-Media)/Desviacion; }

//--------------------------------------------------------------
// Función de densidad de la N(0,1)
function fd(x)
  { return Math.exp(-x*x/2)/Math.sqrt(2*Math.PI); }

//--------------------------------------------------------------
// Valor de la N(0,1) para cualquier x
function n01(x)
  {
  var Respuesta = 0.5;

  if ( x>0 )  { Respuesta = AbramowitzStegun (x); }
  if ( x<0 )  { Respuesta = 1 - AbramowitzStegun (-x); }

  return ( Respuesta );
  }

//--------------------------------------------------------------
// Valor de la N(0,1) para x>0
// Algoritmo de Abramowitz y Stegun
// http://en.wikipedia.org/wiki/Normal_distribution
//        #Numerical_approximations_for_the_normal_CDF
function AbramowitzStegun(x)
  {
  var b0 = 0.2316419;
  var b1 = 0.319381530;
  var b2 = -0.356563782;
  var b3 = 1.781477937;
  var b4 = -1.821255978;
  var b5 = 1.330274429;

  var t = 1 / (1+b0*x);
  var t2 = t*t;
  var t3 = t2*t;
  var t4 = t3*t;
  var t5 = t4*t;

  return ( 1 - fd(x) * (b1*t + b2*t2 + b3*t3 + b4*t4 + b5*t5) );
  }
