//--------------------------------------------------------------
// Fichero:  pimontecarlo.js
// Objetivo: Calcular un valor aproximado de pi usando el método
//           de Montecarlo
// Fecha:    M.14.8.2018
// Autor:    Pedro Reina
// Licencia: CC0 1.0 Universal
//           http://creativecommons.org/publicdomain/zero/1.0/
//--------------------------------------------------------------

//------------------------------------------
// Variables globales
//------------------------------------------

// Datos que proporciona el usuario
var NombreDato = Array();
var NumeroPuntos = null;

// Valores que calcula el programa
var NombreRespuesta = Array ("PiAproximado");
var PiAproximado = null;

// Los controles HTML del método
var InputPiAproximado = null;

// Los controles HTML del interfaz de usuario
var Contexto = null;
var Deslizador = null;

// Los valores de las abscisas y las ordenadas
var ValorAbscisa = Array();
var ValorOrdenada = Array();

//----------------------------
// Funciones que debe implementar el programa
//----------------------------

//--------------------------------------------------------------
// Obtener los identificadores adicionales de los controles HTML
function ObtenIdentificadoresProblema()
  {
  // Averiguamos el id del deslizador
  Deslizador = document.getElementById("deslizador");
  }

//--------------------------------------------------------------
// Colocar el foco en el primer elemento
function FocoAlPrimero()
  { Deslizador.focus(); }

//--------------------------------------------------------------
// Obtener los datos que ha escrito el usuario
function ObtenDatos()
  {  }

//--------------------------------------------------------------
// Comprobación de los datos
function CompruebaDatos()
  { return true; }

//--------------------------------------------------------------
// Calcular las respuestas
function CalculaRespuestas()
  {
  // Borramos los valores anteriores
  ValorAbscisa = Array();
  ValorOrdenada = Array();

  // Obtenemos cuántos puntos hay que utilizar
  NumeroPuntos = Deslizador.value;

  // Las coordenadas de un punto
  var Abscisa;
  var Ordenada;

  // Para contar cuántos puntos caen dentro del círculo
  var PuntosInteriores=0;

  // Trabajamos el número de puntos pedidos
  for ( var i=0 ; i<NumeroPuntos ; i++ )
    {
    // Generamos al azar un punto en el cuadrado [-1,1]x[-1,1]
    Abscisa = 2*Math.random()-1;
    Ordenada = 2*Math.random()-1;
    ValorAbscisa.push(Abscisa);
    ValorOrdenada.push(Ordenada);

    // Vemos si el punto ha caído dentro del círculo de radio 1
    if ( Abscisa*Abscisa + Ordenada*Ordenada < 1 )
      { PuntosInteriores++; }
    }

  // Ya podemos calcular el valor aproximado de pi
  PiAproximado = 4 * PuntosInteriores / NumeroPuntos
  }

//--------------------------------------------------------------
// La realización del dibujo
function RealizaDibujo()
  {
  // La abscisa y la ordenada en coordenadas del canvas
  var Abscisa = null; var Ordenada = null;

  // Calculamos la escala para que se vean bien todos los valores
  Escala = 0.95 * Math.min(AlturaDibujo, AnchuraDibujo) / 2.1

  // El punto central del dibujo
  var OrigenX = AnchuraDibujo/2;
  var OrigenY = AlturaDibujo/2;

  // Trazo medio y color verde para la circunferencia
  var Radio = esc(1);
  Contexto.lineWidth = 2;
  Contexto.strokeStyle = "#00CC00";
  Contexto.beginPath();
  Contexto.arc(OrigenX, OrigenY, Radio, 0, 2*Math.PI, true);
  Contexto.stroke();

  // Trazo medio y color azul para el cuadrado
  Contexto.fillStyle = "#0000CC";
  Contexto.fillRect(OrigenX-Radio, OrigenY-Radio, 2, 2*Radio);
  Contexto.fillRect(OrigenX+Radio, OrigenY-Radio, 2, 2*Radio);
  Contexto.fillRect(OrigenX-Radio, OrigenY-Radio, 2*Radio, 2);
  Contexto.fillRect(OrigenX-Radio, OrigenY+Radio, 2*Radio, 2);

  // Trazamos todos los puntos calculados
  // https://stackoverflow.com/questions/4899799/
  var id = Contexto.createImageData(1,1);
  var d = id.data;
  d[0]=128; d[1]=0; d[2]=0; d[3]=255;
  for ( var i=0 ; i<NumeroPuntos ; i++ )
    {
    Abscisa = OrigenX + esc(ValorAbscisa[i])
    Ordenada = OrigenY + esc(ValorOrdenada[i])
    Contexto.putImageData(id, Abscisa, Ordenada)
    }
  }
