//--------------------------------------------------------------
// Fichero:  progmat.js
// Objetivo: Elementos comunes a Programas de matemáticas
// Fecha:    D.16.7.2017
// Autor:    Pedro Reina <pedro@pedroreina.net>
// Licencia: CC0 1.0 Universal
//           http://creativecommons.org/publicdomain/zero/1.0/
//--------------------------------------------------------------

//------------------------------------------
// Constantes
//------------------------------------------

// Cuánto modificar cada vez el tamaño de la zona de dibujo
var PasoCambioTamano = 20;

// Dimensiones mí­nimas de la zona de dibujo
var MinAnchuraDibujo = 400;
var MinAlturaDibujo = 200;

// Valores máximos admitidos para la pseudoventana
var MinX = 275;
var MinY = 175;

//----------------------------
// Variables globales
//----------------------------

// Controles HTML
var InputAnchuraDibujo = null;
var InputAlturaDibujo = null;
var BotonCalcular = null;
var BotonBorrarResultado = null;
var BotonBorrarTodo = null;
var BotonAncha = null;
var BotonEstrecha = null;
var BotonAlta = null;
var BotonBaja = null;
var Dibujo = null;
var Contexto = null;

// Dimensiones de la zona de dibujo
var AnchuraDibujo = MinAnchuraDibujo;
var AlturaDibujo = MinAlturaDibujo;

// La escala para dibujar
var Escala = 1;

// Manejo de la pseudoventana
var BotonPedir = null;
var DivPseudoventana = null;
var DivPseudotitulo = null;
var DivPseudocontenido = null;
var BotonCerrar = null;
var BotonResize = null;

//------------------------------------------
// Programa principal: cuando se cargue la
// página completa, ejecutamos una función
//------------------------------------------

AgregaEvento(window, 'load', PreparaTodo, false);

//----------------------------
// Funciones
//----------------------------

// Las funciones que enumero a continuación deben ser implementadas
// por los archivos JavaScript de cada programa en concreto, por lo
// que no aparecen en este archivo:
// ObtenIdentificadoresProblema() FocoAlPrimero() ObtenDatos()
// CompruebaDatos() CalculaRespuestas() RealizaDibujo()

//--------------------------------------------------------------
// La función que prepara todo el documento
function PreparaTodo()
  {
  ObtenIdentificadores();
  AgregaEnganches();
  ValoresIniciales();
  FocoAlPrimero();
  PreparaDibujo();
  if ( BotonPedir != null )
    { IniciaArrastrarSoltar(); }
  }

//------------------------------------------
// Funciones para preparar el sistema
//------------------------------------------

//--------------------------------------------------------------
// Obtener los identificadores de los controles HTML
function ObtenIdentificadores()
  {
  // Averiguamos los id de los botones
  BotonCalcular = document.getElementById('Calcular');
  BotonBorrarResultado = document.getElementById('BorrarResultado');
  BotonBorrarTodo = document.getElementById('BorrarTodo');
  BotonAlta = document.getElementById('Alta');
  BotonBaja = document.getElementById('Baja');
  BotonAncha = document.getElementById('Ancha');
  BotonEstrecha = document.getElementById('Estrecha');

  // Averiguamos los id de los inputs de las dimensiones del dibujo
  InputAnchuraDibujo = document.getElementById('AnchDib');
  InputAlturaDibujo = document.getElementById('AltDib');

  // Averiguamos el id del canvas para dibujar
  Dibujo = document.getElementById("Dibujo");

  // Averiguamos los id de los elementos de la pseudoventana
  // Así también sabemos si es necesaria
  BotonPedir = document.getElementById ('pedir');
  if ( BotonPedir != null )
    {
    DivPseudoventana = document.getElementById ('pseudoventana');
    DivPseudotitulo = document.getElementById ('pseudotitulo');
    DivPseudocontenido = document.getElementById ('pseudocontenido');
    BotonCerrar = document.getElementById ('botoncerrar');
    BotonResize = document.getElementById ('botonresize');
    }

  // Averiguamos los id de los input de datos y respuesta
  // Los arrays NombreDato y NombreRespuesta se definen en cada programa
  ObtenIdentificadoresNombre(NombreDato);
  ObtenIdentificadoresNombre(NombreRespuesta);

  // Cada programa puede tener sus propios controles adicionales
  ObtenIdentificadoresProblema();
  }

//--------------------------------------------------------------
// Obtener identificadores de los nombres dados
function ObtenIdentificadoresNombre(Nombre)
  {
  for ( var i=0 ; i < Nombre.length ; i++ )
    { eval ("Input"+Nombre[i]+" = document.getElementById('"+Nombre[i]+"')"); }
  }

//--------------------------------------------------------------
// La función que añade todos los "listeners" necesarios
function AgregaEnganches()
  {
  AgregaEvento (BotonCalcular, 'click', Calcular, false);
  AgregaEvento (BotonBorrarResultado, 'click', BorrarResultado, false);
  AgregaEvento (BotonBorrarTodo, 'click', BorrarTodo, false);
  AgregaEvento (BotonAncha, 'click', MasAncha, false);
  AgregaEvento (BotonEstrecha, 'click', MasEstrecha, false);
  AgregaEvento (BotonAlta, 'click', MasAlta, false);
  AgregaEvento (BotonBaja, 'click', MasBaja, false);
  AgregaEvento (InputAnchuraDibujo, 'change', CambioDimZona, false);
  AgregaEvento (InputAlturaDibujo, 'change', CambioDimZona, false);
  if ( BotonPedir != null )
    {
    AgregaEvento (BotonPedir, 'click', AbrePseudoventana, false);
    AgregaEvento (BotonCerrar, 'click', CierraPseudoventana, false);
    }
  }

//--------------------------------------------------------------
// Valores iniciales en el formulario
function ValoresIniciales()
  {
  // Asignamos a la zona de dibujo el máximo espacio disponible
  var Tamano = Dimensiones();
  var Situacion = Posicion(Dibujo);
  InputAnchuraDibujo.value = Math.max(Tamano[0]-70, MinAnchuraDibujo);
  InputAlturaDibujo.value = Math.max(Tamano[1]-Situacion[1]-10, MinAlturaDibujo);
  }

//--------------------------------------------------------------
// Preparación de la zona para dibujar
function PreparaDibujo()
  {
  // Obtenemos el tamaño de la zona de dibujo
  AnchuraDibujo = parseInt(InputAnchuraDibujo.value);
  AlturaDibujo = parseInt(InputAlturaDibujo.value);

  // Asignamos las dimensiones de la zona para dibujar
  Dibujo.width = AnchuraDibujo;
  Dibujo.height = AlturaDibujo;

  // Obtenemos el contexto de la zona para dibujar
  Contexto = Dibujo.getContext("2d");

  // Marcamos el borde de la zona de dibujo
  Contexto.fillRect (0, 0, AnchuraDibujo, 1);
  Contexto.fillRect (0, 0, 1, AlturaDibujo);
  Contexto.fillRect (AnchuraDibujo-1, 0, 1, AlturaDibujo);
  Contexto.fillRect (0, AlturaDibujo-1, AnchuraDibujo, 1);
  }

//------------------------------------------
// Funciones para manejar el cálculo y dibujo
//------------------------------------------

//--------------------------------------------------------------
// Si se pulsa el botón "Calcular y dibujar"
function Calcular(evento)
  {
  ObtenDatos();

  if ( CompruebaDatos() )
    {
    CalculaRespuestas();
    EscribeRespuestas();
    PreparaDibujo();
    RealizaDibujo();
    }
  }

//--------------------------------------------------------------
// Si se pulsa el botón BorrarTodo, borramos todo
function BorrarTodo()
  {
  // Borramos los datos
  BorraDatos();

  // Borramos el resto
  BorrarResultado();

  // Calculamos los valores iniciales
  ValoresIniciales();

  // Colocamos el foco
  FocoAlPrimero();
  }

//--------------------------------------------------------------
// Si se pulsa el botón BorrarResultado, borramos los resultados
function BorrarResultado(evento)
  {
  // Borramos las respuestas
  BorraRespuestas();

  // Borramos el dibujo

  // 1. Obtenemos las variables de la zona para dibujar
  Contexto = Dibujo.getContext("2d");

  // 2. Borrado real
  Dibujo.width = Dibujo.width;

  // 3. Opera necesita esta llamada
  Contexto.clearRect(0, 0, Dibujo.width, Dibujo.height)

  // 4. Recuperamos las dimensiones mínimas
  Dibujo.width = MinAnchuraDibujo;
  Dibujo.height = MinAlturaDibujo;

  // 5. Dejamos el dibujo como al principio
  PreparaDibujo();
  }

//--------------------------------------------------------------
// Escribir las respuestas
function EscribeRespuestas()
  {
  EscribeTextos(NombreRespuesta);
  Activa(NombreRespuesta);
  }

//--------------------------------------------------------------
// Escribir textos en entradas del formulario
function EscribeTextos(Nombre)
  {
  for ( var i=0 ; i < Nombre.length ; i++ )
    { eval ("Input"+Nombre[i]+".value = "+Nombre[i]); }
  }

//--------------------------------------------------------------
// Borrar los datos
function BorraDatos()
  {
  VariablesACero(NombreDato);
  LimpiaCampo(NombreDato);
  }

//--------------------------------------------------------------
// Borrar las respuestas calculadas
function BorraRespuestas()
  {
  VariablesACero(NombreRespuesta);
  LimpiaCampo(NombreRespuesta);
  Desactiva(NombreRespuesta);
  }

//--------------------------------------------------------------
// Conversión a la escala del dibujo de un dato real
function esc(dato)
  { return Escala*dato; }

//--------------------------------------------------------------
// Activar varias entradas del formulario
function Activa(Nombre)
  {
  for ( var i=0 ; i < Nombre.length ; i++ )
    { eval ("Input"+Nombre[i]+".disabled = false"); }
  }

//--------------------------------------------------------------
// Desactivar varias entradas del formulario
function Desactiva(Nombre)
  {
  for ( var i=0 ; i < Nombre.length ; i++ )
    { eval ("Input"+Nombre[i]+".disabled = true"); }
  }

//--------------------------------------------------------------
// Poner variables a cero
function VariablesACero(Nombre)
  {
  for ( var i=0 ; i < Nombre.length ; i++ )
    { eval (Nombre[i] + " = 0"); }
  }

//--------------------------------------------------------------
// Poner la cadena nula en entradas del formulario
function LimpiaCampo(Nombre)
  {
  for ( var i=0 ; i < Nombre.length ; i++ )
    { eval ("Input"+Nombre[i]+".value = \"\""); }
  }

//--------------------------------------------------------------
// Averiguar si alguna variable no es un número
function NoNumero(Nombre)
  {
  var Respuesta = false;
  var Sigue = true;

  for ( var i=0 ; i < Nombre.length && Sigue ; i++ )
    {
    if ( eval ("isNaN("+Nombre[i]+")") )
      {
      Sigue = false;
      Respuesta = true;
      }
    }

  return Respuesta;
  }

//----------------------------
// Funciones para modificar el
// tamaño de la zona de dibujo
//----------------------------

//--------------------------------------------------------------
// Si nos cambian los input de las dimensiones de la zona
function CambioDimZona()
  {
  // Leemos los valores de los input
  var AnchuraDibujoPropuesta = parseInt(InputAnchuraDibujo.value);
  var AlturaDibujoPropuesta = parseInt(InputAlturaDibujo.value);

  // Los ajustamos si hace falta
  AnchuraDibujo = Math.max(AnchuraDibujoPropuesta, MinAnchuraDibujo);
  AlturaDibujo = Math.max(AlturaDibujoPropuesta, MinAlturaDibujo);

  // Cambiamos el tamaño
  Dibujo.width = AnchuraDibujo;
  Dibujo.height = AlturaDibujo;

  // Volvemos a dibujar
  ActualizaDibujo();
  }

//--------------------------------------------------------------
// Si nos piden hacer más ancha la zona de dibujo
function MasAncha()
  {
  // Cambiamos el tamaño
  AnchuraDibujo += PasoCambioTamano;
  Dibujo.width = AnchuraDibujo;

  // Volvemos a dibujar
  ActualizaDibujo();
  }

//--------------------------------------------------------------
// Si nos piden hacer más estrecha la zona de dibujo
function MasEstrecha()
  {
  // Si aun hay sitio
  if ( AnchuraDibujo > MinAnchuraDibujo )
    {
    // Cambiamos el tamaño
    AnchuraDibujo -= PasoCambioTamano;
    Dibujo.width = AnchuraDibujo;

    // Volvemos a dibujar
    ActualizaDibujo();
    }
  }

//--------------------------------------------------------------
// Si nos piden hacer más alta la zona de dibujo
function MasAlta()
  {
  // Cambiamos el tamaño
  AlturaDibujo += PasoCambioTamano;
  Dibujo.height = AlturaDibujo;

  // Volvemos a dibujar
  ActualizaDibujo();
  }

//--------------------------------------------------------------
// Si nos piden hacer más baja la zona de dibujo
function MasBaja()
  {
  // Si aun hay sitio
  if ( AlturaDibujo > MinAlturaDibujo )
    {
    // Cambiamos el tamaño
    AlturaDibujo -= PasoCambioTamano;
    Dibujo.height = AlturaDibujo;

    // Volvemos a dibujar
    ActualizaDibujo();
    }
  }

//--------------------------------------------------------------
// Si nos piden que actualicemos el dibujo
function ActualizaDibujo()
  {
  InputAlturaDibujo.value = AlturaDibujo;
  InputAnchuraDibujo.value = AnchuraDibujo;
  PreparaDibujo();
  }

//----------------------------
// Funciones JavaScript para manejar
// funciones matemáticas
//----------------------------

//--------------------------------------------------------------
// Cálculo del valor de la función
function f(x)
  { return eval(Expresion); }

//--------------------------------------------------------------
// Modificar la expresión de f(x) para adaptarla a Javascript
function ModificaExpresion()
  {
  Expresion = Expresion.replace (/pot/g, "Math.pow");
  Expresion = Expresion.replace (/abs/g, "Math.abs");
  Expresion = Expresion.replace (/raiz/g, "Math.sqrt");
  Expresion = Expresion.replace (/sen/g, "Math.sin");
  Expresion = Expresion.replace (/arcs/g, "Math.asin");
  Expresion = Expresion.replace (/cos/g, "Math.cos");
  Expresion = Expresion.replace (/arcc/g, "Math.acos");
  Expresion = Expresion.replace (/tan/g, "Math.tan");
  Expresion = Expresion.replace (/arct/g, "Math.atan");
  Expresion = Expresion.replace (/ln/g, "Math.log");
  Expresion = Expresion.replace (/exp/g, "Math.exp");
  }

//----------------------------
// Funciones para abrir y cerrar
// la pseudoventana
//----------------------------

//--------------------------------------------------------------
// Hace que aparezca la pseudoventana
function AbrePseudoventana()
  { DivPseudoventana.style.display = "block"; }

//--------------------------------------------------------------
// Hace que desaparezca la pseudoventana
function CierraPseudoventana()
  { DivPseudoventana.style.display = "none"; }

//----------------------------
// Funciones para arrastar y soltar
// http://luke.breuer.com/tutorial/javascript-drag-and-drop-tutorial.aspx
//----------------------------

//--------------------------------------------------------------
// Inicia el sistema de arrastrar y soltar
function IniciaArrastrarSoltar()
  {
  // Asignamos un nuevo tratamiento a dos eventos de ratón
  document.onmousedown = OnMouseDown;
  document.onmouseup = OnMouseUp;
  }

//--------------------------------------------------------------
// Pulsan el botón del ratón
function OnMouseDown(e)
  {
  // IE is retarded and doesn't pass the event object
  if ( e == null )
    { e = window.event; }

  // Buscamos el elemento sobre el que han pulsado
  // IE uses srcElement, others use target
  var Elemento = e.target != null ? e.target : e.srcElement;

  // Vemos si han pulsado en una zona en que dejamos arrastrar
  ZonasArrastrables = ['pseudotitulo', 'botonresize'];
  ElementoEsArrastrable = (ZonasArrastrables.indexOf(Elemento.className)!=-1);

  // for IE, left click == 1
  // for Firefox, left click == 0
  if ( (e.button == 1 && window.event != null || e.button == 0)
       && ElementoEsArrastrable )
    {
    // Nos guardamos la posición del ratón
    ComienzoX = e.clientX;
    ComienzoY = e.clientY;

    // Nos guardamos la posición y tamaño de la pseudoventana y sus elementos
    DesplazamientoX = ExtraeNumero (DivPseudoventana.style.left);
    DesplazamientoY = ExtraeNumero (DivPseudoventana.style.top);
    TamanoX = ExtraeNumero (DivPseudoventana.style.width);
    TamanoY = ExtraeNumero (DivPseudoventana.style.height);
    TamanoTituloX = ExtraeNumero (DivPseudotitulo.style.width);
    TamanoContenidoX = ExtraeNumero (DivPseudocontenido.style.width);
    TamanoContenidoY = ExtraeNumero (DivPseudocontenido.style.height);

    // Llevamos el elemento muy al frente mientras lo arrastramos
    zIndexAnterior = DivPseudoventana.style.zIndex;
    DivPseudoventana.style.zIndex = 10000;

    // Cambiamos el cursor para indicar que estamos arrastrando
    if ( Elemento == DivPseudotitulo )
      { Elemento.style.cursor = "grabbing"; }

    // Nos guardamos el elemento para usarlo en OnMouseMove
    ElementoArrastrado = Elemento;

    // Ahora modificaremos el elemento cuando muevan el ratón
    document.onmousemove = OnMouseMove;

    // No permitimos que se seleccione texto mientras arrastramos
    document.body.focus();

    // Tampoco lo permitimos en IE
    document.onselectstart = function () { return false; };

    // Tampoco permitimos a IE que se arrastre una imagen
    Elemento.ondragstart = function() { return false; };

    // prevent text selection (except IE)
    return false;
    }
  }

//--------------------------------------------------------------
// Mueven el ratón
function OnMouseMove(e)
  {
  // IE is retarded and doesn't pass the event object
  if (e == null)
    { var e = window.event; }

  // Calculamos cuánto se ha movido el ratón desde su posición inicial
  AvanceX = e.clientX - ComienzoX;
  AvanceY = e.clientY - ComienzoY;

  // Este es el código que realmente arrastra la pseudoventana
  if ( ElementoArrastrado == DivPseudotitulo )
    {
    DivPseudoventana.style.left = (DesplazamientoX + AvanceX) + 'px';
    if ( DesplazamientoY + AvanceY > 0 )
      { DivPseudoventana.style.top = (DesplazamientoY + AvanceY) + 'px'; }
    }

  // Este es el código que realmente modifica la pseudoventana
  if ( ElementoArrastrado == BotonResize )
    {
    if ( TamanoX + AvanceX >= MinX )
      {
      DivPseudoventana.style.width = (TamanoX + AvanceX) + 'px';
      DivPseudotitulo.style.width = (TamanoTituloX + AvanceX) + 'px';
      DivPseudocontenido.style.width = (TamanoContenidoX + AvanceX) + 'px';
      }
    if ( TamanoY + AvanceY >= MinY )
      {
      DivPseudoventana.style.height = (TamanoY + AvanceY) + 'px';
      DivPseudocontenido.style.height = (TamanoContenidoY + AvanceY) + 'px';
      }
    }
  }

//--------------------------------------------------------------
// Sueltan el ratón
function OnMouseUp(e)
  {
  // Comprobamos que realmente estamos arrastrando un elemento
  if ( ElementoArrastrado != null )
    {
    // Devolvemos el zIndex al elemento
    DivPseudoventana.style.zIndex = zIndexAnterior;

    // Cambiamos el cursor para indicar que se podrá volver a mover
    if ( ElementoArrastrado == DivPseudotitulo )
      { ElementoArrastrado.style.cursor = "move"; }

    // Hemos terminado con estos eventos hasta el próximo arrastre
    document.onmousemove = null;
    document.onselectstart = null;
    ElementoArrastrado.ondragstart = null;

    // Anotamos que ya no estamos arrastrando
    ElementoArrastrado = null;
    }
  }

//--------------------------------------------------------------
// Convierte un valor (que puede ser NaN) en un número
function ExtraeNumero (value)
  {
  var n = parseInt (value);
  return ( n == null || isNaN(n) ) ? 0 : n;
  }

//----------------------------
// Utilidades JavaScript
//----------------------------

//--------------------------------------------------------------
// Obtención de la anchura y altura disponibles
// http://www.howtocreate.co.uk/tutorials/javascript/browserwindow
function Dimensiones()
  {
  var myWidth = 0, myHeight = 0;

  if ( typeof (window.innerWidth) == 'number' )
    {
    // Non-IE
    myWidth = window.innerWidth;
    myHeight = window.innerHeight;
    }

  else if ( document.documentElement &&
          ( document.documentElement.clientWidth ||
            document.documentElement.clientHeight ) )
    {
    // IE 6+ in 'standards compliant mode'
    myWidth = document.documentElement.clientWidth;
    myHeight = document.documentElement.clientHeight;
    }

  else if ( document.body &&
          ( document.body.clientWidth || document.body.clientHeight ) )
    {
    // IE 4 compatible
    myWidth = document.body.clientWidth;
    myHeight = document.body.clientHeight;
    }

  return Array (myWidth, myHeight);
  }

//--------------------------------------------------------------
// Obtención de la posición de un elemento
// http://stackoverflow.com/questions/442404/
//      dynamically-retrieve-html-element-x-y-position-with-javascript
function Posicion(el)
  {
  var _x = 0;
  var _y = 0;

  while ( el && !isNaN (el.offsetLeft) && !isNaN (el.offsetTop) )
    {
    _x += el.offsetLeft - el.scrollLeft;
    _y += el.offsetTop - el.scrollTop;
    el = el.offsetParent;
    }

  return Array (_x, _y);
  }

//----------------------------
// Manejo de eventos
//----------------------------

// Técnicas explicadas en http://www.sitepoint.com/books/dhtml1/

//--------------------------------------------------------------
// Función portable que añade un evento
function AgregaEvento (elemento, evType, funcion, useCapture)
  {
  // Manejador de eventos compatible con navegadores IE5+, NS6 y Mozilla
  // Autor: Scott Andrew
  if ( elemento.addEventListener )
    {
    elemento.addEventListener (evType, funcion, useCapture);
    return true;
    }
  else if ( elemento.attachEvent )
    {
    var r = elemento.attachEvent ('on' + evType, funcion);
    return r;
    }
  else
    { elemento ['on' + evType] = funcion; }
  }
